@@ -24,7 +24,10 @@ def convert_to_datetime(date: Optional[str]) -> Optional[datetime]:
 
 @shared_task(name=__name__ + ".LoadCvsRepository")
 def load_cvs(
-    *, url: str, origin_url: Optional[str] = None, visit_date: Optional[str] = None,
+    *,
+    url: str,
+    origin_url: Optional[str] = None,
+    visit_date: Optional[str] = None,
 ):
     """Import a CVS repository
 
@@ -35,6 +38,8 @@ def load_cvs(
         - visit_date: Optional date to override the visit date
     """
     loader = CvsLoader.from_configfile(
-        url=url, origin_url=origin_url, visit_date=convert_to_datetime(visit_date),
+        url=url,
+        origin_url=origin_url,
+        visit_date=convert_to_datetime(visit_date),
     )
     return loader.load()
